function [G1,C,impact,eu,SDX,zmat,rbar,B,coe_C,coe_Z,Br,GAM0,PSI]=model_full4(param,coe_C,coe_Z)

dd=[];

RHO=.75;

% -------------------------------------------------------------------------
% INDEX for endogenous variables
% -------------------------------------------------------------------------
varn=0;
for var={
    'y',...            % 1 output
    'k',...            % 2 capital
    'L',...            % 3 hours
    'Rk',...           % 4 return on capital
    'w',...            % 5 real wages
    'p',...            % 6 inflation
    's',...            % 7 marginal cost
    'c',...            % 8 consumption
    'R',...           % 9 nominal interst rate
    'I',...           % 10 final investment production
    'x',...           % 11 Investment expenditures
    'pk',...          % 12 price of capital goods
    'wgap',...
    'eRkb',...          
    ... %%% shocks
    'a',...           % 15 productivity
    'g',...           % gov't spending
    'miu',...         % investment shock
    'lambdap',...      % 18 price markup shock
    'lambdaw',...     % wage markup shock
    'u',...           % intertemporal preference shock 
    'mp',...          % monetary policy shock
    'PISTAR',...      % inflation target
    'RRA_Z',...       % 23 risk aversion
    'ARMAlambdap',...
    'ARMAlambdaw',...
    ... %%% expectational variables
... %     'epdelta',...
    'ep',...          % 26
    'ec',...
    'Rkb',...           % 28
    'ex',...           
    'ew',...        
    'c_1',...        % 31
    'k_1',...        % 32
    'y_1',...
    'x_1',...         
    'w_1',...       
    'RU',...        
    'WC',...
    'eRU',...       % 38
    'U',...
    'UC',...
    'eUC',...       
...%
    'y_2',...       % 50
    'y_3',...
    'y_4',...
    'epspi',...
    'aMA',...
    'sh_1',...
    'sh_2',...
    'sh_3',...
    'sh_4',...
    'sh_5',...
    'sh_6',...
    'sh_7',...    
    'sh_8',...
    'sh_9',...
    'ea',...
    'CONS'}         
    varn=varn+1;
    eval([var{1} '=varn;']);
end

NY=varn;   %% need 71 equations

% -------------------------------------------------------------------------
% INDEX for Shocks  
% -------------------------------------------------------------------------
Rs=1;           % Monetary policy
as=2;           % Technology 
gs=3;           % Government spending
mius=4;         % Investment     
lambdaps=5;     % Price markup
lambdaws=6;     % Wage markup
us=7;           % Intertemporal preference
PISs=8;         % inflation target
Zs=9;           % risk aversion

NX=9;


% -------------------------------------------------------------------------
% INDEX for expectational errors
% -------------------------------------------------------------------------

varnex=0;
for var={
    'pex',... 
    'cex',...
    'xex',...
    'wex',...
    'aex',...
    'RUex',...
    'UCex',...
    'aex',...
    'Rkbex'}

    varnex=varnex+1;
    eval([var{1} '=varnex;']);
end

NETA=varnex;


% -------------------------------------------------------------------------
% Index for the parameters 
% -------------------------------------------------------------------------

% calibrated parameters
gss=0.22;           % steady state government spending to GDP ratio


% Non SD parameters 
% -------------------------------------------------------------------------
alpha=.33;
iotap=param(1);     % price indexation (=0 is static indexation, =1 is dynamic)
iotaw=param(2);     % wages indexation
gamma100=.47;  % steady state growth rate of technology (multiplied by 100)
ETA=param(3);         % habit formation 
lambdapss=.2; % steady state net price markup
lambdawss=.05; % steady state net wage markup
Lss=6.75;       % steady state for log hours
pss100=2.5/4;    % steady state quarterly net inflation (multiplied by 100)
Fbeta=0.35;    % 100 * ( inv(discount factor) - 1 )
niu=1;      % inverse Frisch elasticity
xip=.66;      % price stickiness
xiw=.66;      % wage stickiness
S=param(5);        % investment adjustment cost
fp=param(6);       % reaction to inflation
fy=param(7);       % reaction to output gap
fdy=param(8);      % reaction to output gap growth
rhoR=param(9); rhoz=param(10); rhog=param(11); rhomiu=param(12); rholambdap=param(13); rholambdaw=param(14); rhob=param(15);
rho_Z=param(16);
rhomp=0;

PHI=1-.0132/4; PHIss=PHI;
delta=.0275;
% S=0;


%% Frisch elasticity
alphaH=.66667;
FRISCH=niu;
% maximum hours
Hbar=exp(Lss)*(1-alphaH*(1-RHO)+FRISCH)/FRISCH;


% Standard deviations 
% -------------------------------------------------------------------------
param(17:22)=param(17:22)/100;  %% put SD parameters into the correct scaling (note b doesn't get divided)
sdR=param(17); sdz=param(18); sdg=param(19); sdmiu=param(20); sdlambdap=param(21); sdlambdaw=param(22); sdb=param(23);

rhoARMAlambdap=param(31);
rhoARMAlambdaw=param(32);

sd_PIS=param(24)/100;
rho_PIS=.99^(1/4);

sd_Z=param(25)/(1-RHO);
ALPHA=param(28);
Z_bar=(1-ALPHA)/(1-RHO);

% sd_sige=param(34);
                  
SDX=diag([sdR sdz sdg sdmiu sdlambdap sdlambdaw sdb sd_PIS sd_Z]);
%%% now allow shocks to z to be correlated with the other shocks

%% param(43:48) gives correlations of the shocks with sd_z
% corr(MP,z)=param(43)=cov(MP,z)/sqrt(sdR*sdz)
% MP=eps_mp+a_mp*eps_z
% cov(MP,z)=a_mp*var(eps_z)
% a_mp*var(eps_z)/(sdR*sdz)=param(43)
% a_mp=param(43)*(sdR*sdz)/(sd_z^2)=param(43)*sdR/sdz;

SDX(8,2)=param(27)/100;
SDX(8,4)=param(4)/100;
SDX(8,9)=param(33)/100;

% -------------------------------------------------------------------------
% Computation of the steady state
% -------------------------------------------------------------------------
gamma=gamma100/100;
beta=100/(Fbeta+100);
LAMss=exp(-RHO*gamma)*beta;
rbar=1/LAMss;
IK=1-(1-delta)*exp(-gamma);

rkbss=(1/LAMss);
rkss=rkbss-(1-delta);

%%%%%%%%%%%%%%%%%%%%%%%
pss=pss100/100;
gss=1/(1-gss);

expLss=exp(Lss);
Rkss=rkss;
sss=1/(1+lambdapss);
wss=(sss*((1-alpha)^(1-alpha))/((alpha^(-alpha))*Rkss^alpha))^(1/(1-alpha));
kLss=exp(gamma)*(wss/Rkss)*alpha/(1-alpha);
FLss=(exp(-gamma*alpha)*kLss^alpha-Rkss*kLss-wss);
yLss=exp(-gamma*alpha)*(kLss^alpha)-FLss;
kss=kLss*expLss;
iss=(1-(1-delta)*exp(-gamma))*kss;
F=FLss*expLss;
yss=yLss*expLss;
css=yss/gss-iss;

phi1 = ETA*wss*(css^(-RHO))/...
        ( alphaH * ((Hbar-exp(Lss))^(alphaH*(1-RHO)))  );
zc1 = ETA*(css^(1-RHO))/...
    ( (css^(1-RHO))/(1-RHO) + phi1*( (Hbar-exp(Lss))^(alphaH*(1-RHO)) )/(1-RHO) );
zc2 = (1-ETA)*(css^(1-RHO))/...
    ( (css^(1-RHO))/(1-RHO) + phi1*( (Hbar-exp(Lss))^(alphaH*(1-RHO)) )/(1-RHO) );
zn = -alphaH*exp(Lss)*phi1 * ((Hbar-exp(Lss))^(alphaH*(1-RHO)))/...
    ( (css^(1-RHO))/(1-RHO) + phi1*( (Hbar-exp(Lss))^(alphaH*(1-RHO)) )/(1-RHO) );

%%%
% log(U/Z^(1-rho)) = zc1*c + zc2*c(-1) + zc2*z + zn*n -- note these are all scaled variables
% log(U_c/Z^(-rho)) = ((ETA)*(1-RHO)-1)*c + (1-ETA)*(1-RHO)*c(-1) + (ETA-1)*(1-RHO)*z -- also all scaled variables
% so anywhere that we have the change in U or U_C, we need to include something involving z
%%%

% -------------------------------------------------------------------------
% System Matrices 
% -------------------------------------------------------------------------
GAM0 = zeros(NY,NY) ;
GAM1 = zeros(NY,NY) ;
PSI = zeros(NY,NX) ;
PPI = zeros(NY,NETA) ;
C = zeros(NY,1) ;


% eq 1 and 2, production function (y, ystar)
% -------------------------------------------------------------------------
% same as with no FF
GAM0(y,y)=1;
GAM0(y,k_1)=-((yss+F)/yss)*alpha;
GAM0(y,L)=-((yss+F)/yss)*(1-alpha);
GAM0(y,a)=((yss+F)/yss)*alpha;

% eq 3 and 4, cost minimization (L and Lstar)
% -------------------------------------------------------------------------
% same as with no FF
GAM0(L,Rk)=1;
GAM0(L,w)=-1;
GAM0(L,L)=-1;
GAM0(L,a)=-1;
GAM0(L,k_1)=1;


% eq 5 and 6, marginal cost (s and sstar)
% -------------------------------------------------------------------------
% same as with no FF
GAM0(s,s)=1;
GAM0(s,Rk)=-alpha;
GAM0(s,w)=-(1-alpha);

% eq 7 and 8, Phillips curve (p and Rstar)
% -------------------------------------------------------------------------
% same as with no FF
GAM0(p,p)=1;
GAM0(p,ep)=-LAMss/(1+iotap*LAMss);
GAM1(p,p)=iotap/(1+iotap*LAMss);
GAM0(p,s)=-((1-LAMss*xip)*(1-xip)/((1+iotap*LAMss)*xip));
GAM0(p,lambdap)=-1;        


% eq 9 and 10, consumption foc (c and cstar)
% -------------------------------------------------------------------------
% same as with no FF
GAM0(c,R)=1;
GAM0(c,eRU)=-1;
GAM0(c,ep)=-1;


% eq 13 and 14, definition of capital return
% -------------------------------------------------------------------------
% same as with no FF
GAM0(Rkb,Rkb)=-rkbss;
GAM0(Rkb,Rk)=rkss;
GAM0(Rkb,pk)=(1-delta); %% what happens when this is equal to 1?
GAM1(Rkb,pk)=rkbss;
% GAM0(Rkb,p)=rkbss;

% same as with no FF when xi=0
GAM0(I,R)=1;
GAM0(I,ep)=-1;
GAM0(I,eRkb)=-1;


% eq 19 and 20, capital accumulation (kbar and kbarstar)
% -------------------------------------------------------------------------
GAM0(k,k)=1;        
GAM0(k,I)=-(1-(1-delta)*exp(-gamma));
GAM1(k,k)=(1-delta)*exp(-gamma);
GAM0(k,a)=(1-delta)*exp(-gamma);


% eq 21 and 22, investment definition
% -------------------------------------------------------------------------
% same as with no FF
GAM0(x,I)=1;
GAM0(x,x)=-1;
GAM0(x,miu)=-1;

% eq 23 and 24, capital good prices
% -------------------------------------------------------------------------

% Adjustment costs in differences
GAM0(pk,pk)=1;
GAM0(pk,miu)=1;
GAM0(pk,x)=-LAMss*S*exp(3*gamma)-S*exp(2*gamma);
GAM1(pk,x)=-S*exp(2*gamma);
GAM0(pk,a)=-S*exp(2*gamma);
GAM0(pk,ea)=LAMss*S*exp(3*gamma);
GAM0(pk,ex)=LAMss*S*exp(3*gamma);

% Adjustment costs in levels
% GAM0(pk,pk)=1;
% GAM0(pk,miu)=1;
% GAM0(pk,x)=-S;
% GAM0(pk,k)=S;


% eq 25, monetary policy rule (R) (taylor rule)
% ----------------------------------------
% same as with no FF
GAM0(R,R)=1;
GAM1(R,R)=rhoR;
GAM0(R,p)=-(1-rhoR)*fp;

GAM0(R,y)=-(1-rhoR)*fy-fdy/4;
GAM0(R,y_4)=fdy/4;

GAM0(R,mp)=-1;
GAM0(R,PISTAR)=-(1-rhoR)+(1-rhoR)*fp;

GAM0(R,CONS)=param(30);

% R = rhoR*R(-1) + (1-rhoR)*fp*(p-pistar) + (1-rhoR*fy*y + (fdy/4)*(y-y(-4))
%     + mp + (1-rhoR)*pistar - param(33)


% eq 28 and 29, market clearing 
% -------------------------------------------------------------------------
% same as with no FF
GAM0(Rk,c)=css/yss;
GAM0(Rk,x)=iss/yss;
GAM0(Rk,y)=-1/gss;
GAM0(Rk,g)=1/gss;

% eq 30,31, wage gap
% -------------------------------------------------------------------------
% same as with no FF
GAM0(wgap,wgap)=1;
GAM0(wgap,w)=-1;
GAM0(wgap,L)=FRISCH;
GAM0(wgap,c)=-(ETA*(1-RHO)-1);
GAM1(wgap,c)=(1-ETA)*(1-RHO);
GAM0(wgap,a)=(1-ETA)*(1-RHO);

% eq 21 and 22, wage Phillips curve (w and wstar)
% -------------------------------------------------------------------------
% same as with no FF
kappaw=[(1-xiw*beta)*(1-xiw)]/[xiw*(1+beta)*(1+FRISCH*(1+1/lambdawss))];
GAM0(w,w)=1;
GAM0(w,ew)=-beta/(1+beta);
GAM0(w,wgap)=kappaw;
GAM0(w,p)=(1+beta*iotaw)/(1+beta);
GAM0(w,ep)=-beta/(1+beta);
GAM0(w,a)=(1+beta*iotaw)/(1+beta);
GAM0(w,ea)=(-beta)/(1+beta);
GAM0(w,lambdaw)=-1;
GAM1(w,w)=1/(1+beta);
GAM1(w,p)=iotaw/(1+beta);
GAM1(w,a)=iotaw/(1+beta);

% -------------------------------------------------------------------------
% -------------------------------------------------------------------------
%%% extra equations for the SDF
% -------------------------------------------------------------------------
% -------------------------------------------------------------------------

% eq 30 and 31, Euler equation for the utility return (RU and RUstar)
% -------------------------------------------------------------------------

betaI=1/(1-beta);

GAM0(RU,eRU) = 1;
GAM0(RU,eUC) = 1;
GAM0(RU,UC) = -1;
GAM0(RU,ea) = -RHO; %% UC is scaled by Z^(-rho)

% eRU = -(eUC-UC) + beta*betaI*eB - betaI*b + RHO*Ez
% eRU = -(eUC-UC - RHO*Ez) + beta*betaI*Eb - betaI*b

% eq 32 and 33, Definition of the utility return (WC and WCstar)
% -------------------------------------------------------------------------

rbar=-log(beta) + RHO*gamma;
% wbar=-log(exp(rbar-gamma)-1);
wbar=log(1/(.025/4));
theta = (exp(wbar)+1)/exp(wbar);


GAM0(WC,RU) = theta;
GAM0(WC,U) = -theta;
GAM0(WC,UC) = theta;
GAM0(WC,a) = -theta; %% U/UC is scaled by A, so we add a anywhere we have a d(U/UC) we need an a
GAM0(WC,WC)=-1;

GAM1(WC,U)=-theta;
GAM1(WC,UC)=theta;
GAM1(WC,WC)=-theta;


% eq 34-37, Definition of the utility and marginal utility (U, UC, Ustar,
% UCstar)
% -------------------------------------------------------------------------

%%%%%%%%%%%%%%%%
% log(U/Z^(1-rho)) = zc1*c + zc2*c(-1) - zc2*z + zn*n -- note these are all scaled variables
% log(U_c/Z^(-rho)) = ((ETA)*(1-RHO)-1)*c + (1-ETA)*(1-RHO)*c(-1) + (ETA-1)*(1-RHO)*z -- also all scaled variables
% so anywhere that we have the change in U or U_C, we need to include something involving z
%%%

GAM0(U,U)=-1;
GAM0(U,c) = zc1;
GAM0(U,c_1) = zc2;
GAM0(U,a) = -zc2;
GAM0(U,L) = zn;

GAM0(UC,UC)=-1;
GAM0(UC,c) = ETA*(1-RHO)-1;
GAM0(UC,c_1) = (1-ETA)*(1-RHO);
GAM0(UC,a) = (ETA-1)*(1-RHO);

% eq 38 - 48, exogenous shocks
% -------------------------------------------------------------------------

% GAM0(a,a)=1; GAM1(a,a)=rhoz; PSI(a,as)=1;

% MA(1)
GAM0(a,a)=1; GAM0(a,aMA)=-1; GAM1(a,aMA)=-rhoz;
GAM0(aMA,aMA)=1; PSI(aMA,as)=1;

GAM0(g,g)=1; GAM1(g,g)=rhog; PSI(g,gs)=1;
GAM0(miu,miu)=1; GAM1(miu,miu)=rhomiu; PSI(miu,mius)=1;
GAM0(mp,mp)=1; GAM1(mp,mp)=rhomp; PSI(mp,Rs)=1;
% GAM0(sigx,sigx)=1; GAM1(sigx,sigx)=rhosig; PSI(sigx,sigs)=1;

GAM0(u,u)=1; GAM1(u,u)=rhob; PSI(u,us)=1; 
GAM0(u,CONS)=param(26);


GAM0(lambdaw,lambdaw)=1; 
GAM1(lambdaw,lambdaw)=rholambdaw; 
GAM0(lambdaw,ARMAlambdaw)=-1; 
GAM1(lambdaw,ARMAlambdaw)=-rhoARMAlambdaw;
GAM0(ARMAlambdaw,ARMAlambdaw)=1; 
PSI(ARMAlambdaw,lambdaws)=1;

%%% AR(1)
% GAM0(lambdap,lambdap)=1; 
% GAM0(lambdap,ARMAlambdap)=-1; 
% GAM1(lambdap,lambdap)=rholambdap;
% GAM0(ARMAlambdap,ARMAlambdap)=1; 
% PSI(ARMAlambdap,lambdaps)=1;

%%% ARMA(1,1)
GAM0(lambdap,lambdap)=1; 
GAM1(lambdap,lambdap)=rholambdap; 
GAM0(lambdap,ARMAlambdap)=-1; 
GAM1(lambdap,ARMAlambdap)=-rhoARMAlambdap;
GAM0(ARMAlambdap,ARMAlambdap)=1; 
PSI(ARMAlambdap,lambdaps)=1;

% GAM0(PISTAR,PISTAR)=1; 
% GAM1(PISTAR,PISTAR)=rho_PIS;
% GAM0(PISTAR,ARMApis)=-1; 
% GAM1(PISTAR,ARMApis)=-param(43);
% GAM0(ARMApis,ARMApis)=1; PSI(ARMApis,8)=1;

%% inflation target
%pi*(t) = rho*pi*(t-1) + gamma*(pi(t-1)-pi*(t-1)) + eps_pi*
GAM0(PISTAR,PISTAR)=1;
GAM1(PISTAR,PISTAR)=rho_PIS;
GAM0(PISTAR,epspi)=-1;

% param(33)=0;
GAM0(epspi,epspi)=1;
PSI(epspi,8)=1;

%% risk aversion
GAM0(RRA_Z,RRA_Z)=1;
GAM1(RRA_Z,RRA_Z)=rho_Z;
GAM0(RRA_Z,CONS)=-(1-rho_Z)*Z_bar;
PSI(RRA_Z,9)=1;

% ARMA(1,1)
% GAM0(RRA_Z,RRA_Z)=1; 
% GAM1(RRA_Z,RRA_Z)=rho_Z; 
% GAM0(RRA_Z,ARMArra)=-1; 
% GAM1(RRA_Z,ARMArra)=-param(34);
% GAM0(ARMArra,ARMArra)=1; 
% PSI(ARMArra,9)=1;
% GAM0(RRA_Z,CONS)=-(1-rho_Z)*Z_bar;

% eq 49 - 62, expectational terms
% -------------------------------------------------------------------------
GAM0(ep,p)=1; GAM1(ep,ep)=1; PPI(ep,pex)=1;
GAM0(ea,a)=1; GAM1(ea,ea)=1; PPI(ea,aex)=1;
GAM0(ec,c)=1; GAM1(ec,ec)=1; PPI(ec,cex)=1;
GAM0(ex,x)=1; GAM1(ex,ex)=1; PPI(ex,xex)=1;
GAM0(ew,w)=1; GAM1(ew,ew)=1; PPI(ew,wex)=1;
GAM0(eRkb,Rkb)=1; GAM1(eRkb,eRkb)=1; PPI(eRkb,Rkbex)=1;

GAM0(eRU,RU)=1; GAM1(eRU,eRU)=1; PPI(eRU,RUex)=1;
GAM0(eUC,UC)=1; GAM1(eUC,eUC)=1; PPI(eUC,UCex)=1;

% eq 63 - 71, lagged variables
%--------------------------------------------------------------------------
GAM0(c_1,c_1)=1; GAM1(c_1,c)=1;
GAM0(k_1,k_1)=1; GAM1(k_1,k)=1;
GAM0(x_1,x_1)=1; GAM1(x_1,x)=1;
GAM0(w_1,w_1)=1; GAM1(w_1,w)=1;

GAM0(y_1,y_1)=1; GAM1(y_1,y)=1;
GAM0(y_2,y_2)=1; GAM1(y_2,y_1)=1;
GAM0(y_3,y_3)=1; GAM1(y_3,y_2)=1;
GAM0(y_4,y_4)=1; GAM1(y_4,y_3)=1;

GAM0(CONS,CONS)=1; GAM1(CONS,CONS)=1;

GAM0(sh_1,sh_1)=1;
 PSI(sh_1,1)=1;
GAM0(sh_2,sh_2)=1;
 PSI(sh_2,2)=1;
GAM0(sh_3,sh_3)=1;
 PSI(sh_3,3)=1;
GAM0(sh_4,sh_4)=1;
 PSI(sh_4,4)=1;
GAM0(sh_5,sh_5)=1;
 PSI(sh_5,5)=1;
GAM0(sh_6,sh_6)=1;
 PSI(sh_6,6)=1;
GAM0(sh_7,sh_7)=1;
 PSI(sh_7,7)=1;
GAM0(sh_8,sh_8)=1;
 PSI(sh_8,8)=1;
GAM0(sh_9,sh_9)=1;
 PSI(sh_9,9)=1;
 
% Solution of the RE system of equations using Chris Sims' Gensys
% -------------------------------------------------------------------------

    NY=size(GAM1,1);
%     SS=(G1^10000)*[ones(NY-1,1);1];

% -------------------------------------------------------------------------
% -------------------------------------------------------------------------
% -------------------------------------------------------------------------
% iteration to get responses to risk aversion
% -------------------------------------------------------------------------
% -------------------------------------------------------------------------
% -------------------------------------------------------------------------
    coe_Z=zeros(1,3);
    coe_C=zeros(1,3);

Cmax=550;

    
    GAM0([RU,I,c],:)=...
        0*GAM0([RU,I,c],:);
    % -------------------------------------------------------------------------
    % parts of the equations that don't involve risk premia
    % (so that we don't have to keep rewriting these guys)
    % -------------------------------------------------------------------------
    % -------------------------------------------------------------------------
    
    % wealth return
    % -------------------------------------------------------------------------
    GAM0(RU,eRU) = 1;
    GAM0(RU,eUC) = 1;
    GAM0(RU,UC) = -1;
    GAM0(RU,ea) = -RHO;
    GAM0(RU,RRA_Z)=coe_Z(1);
    GAM0(RU,CONS)=coe_C(1);

    % Risk-free rate
    % -------------------------------------------------------------------------    
    GAM0(c,R)=1;
    GAM0(c,eRU)=-1;
    GAM0(c,u)=1;
    GAM0(c,ep)=-1;
    GAM0(c,RRA_Z)=coe_Z(2);
    GAM0(c,CONS)=coe_C(2);
    
%     R-ep=eRU-u-coe_Z(2)*Z
%     R-ep=eRU-(param(28)-1)*(coe_Z(1)+coe_Z(2))-coe_Z(2)*Z
    
    
    % Tobin's q
    % -------------------------------------------------------------------------
    GAM0(I,eRU)=-1;
    GAM0(I,u)=1;
    GAM0(I,eRkb)=1;
    GAM0(I,RRA_Z)=coe_Z(3);
    GAM0(I,CONS)=coe_C(3);
    
    
        CC=zeros(1,5);
        iter1=1;
        iter2=1;
        while iter1<31 & iter2>1e-20
            CCnew=tester14_bonds([0 CC],GAM0,GAM1,C,PSI,PPI,SDX);
            iter2=sum((1-CCnew./CC).^2);
            CC=CCnew;
            iter1=iter1+1;
            if sum(CC)>3;
                iter1=32;
                iter2=10;
            end
        end

        if iter2>1e-20 & iter2<1
        [CC,~,EX]=fsolve(@(x) -1+x./tester14_bonds([0 x],GAM0,GAM1,C,PSI,PPI,SDX),CC,...
            optimset('display','off','tolfun',1e-40,'tolx',1e-10,...
            'algorithm',{'levenberg-marquardt',.01},'maxiter',25));
        elseif iter2<1e-20; 
            EX=4;
        else
            EX=-4;
        end
% EX=1;
CC=[0 CC];
coe_C=CC(1:3); coe_Z=CC(4:6);
%         GAM0(u,RRA_Z) = -(param(28)-1)*(coe_Z(1)+coe_Z(2));

        GAM0(RU,RRA_Z)=coe_Z(1);
        GAM0(RU,CONS)=coe_C(1);

        GAM0(I,RRA_Z)=coe_Z(3);
        GAM0(I,CONS)=coe_C(3);
        
        GAM0(c,RRA_Z)=coe_Z(2);
        GAM0(c,CONS)=coe_C(2);
        
        [G1,C,impact,fmat,fwt,ywt,gev,eu]=gensys(GAM0,GAM1,C,PSI,PPI);
        
    
      PPSI=impact;


 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 if sum(eu)==2
        PHI=G1;
        
        NY=length(G1);

        G_R = zeros(1,NY); G_R(RU)=1;
        G_UC = zeros(1,NY); G_UC(UC)=1; G_UC(a)=-RHO;
        G_b = zeros(1,NY); G_b(u)=1;
        G_pi = zeros(1,NY); G_pi(p)=1;
        G_z = zeros(1,NY); G_z(RRA_Z)=1;
        
%         SDX2=0*SDX;
%         SDX2(:,2)=SDX(:,2);
%         SDX2(:,9)=SDX(:,9);
        SDX2=SDX;
        
        A=zeros(41,1); B=zeros(41,NY);
        for i=1:400
            A(i+1) = -rbar + A(i) + (1/2)*(-G_R + B(i,:) - G_pi)*PPSI*SDX2*SDX2'*PPSI'*(-G_R + B(i,:) - G_pi)';
            B(i+1,:) = -G_R*PHI + G_b + B(i,:)*PHI - G_pi*PHI...
                + (G_UC+G_R)*PPSI*SDX2*SDX2'*PPSI'*(-G_R+B(i,:)-G_pi)'*G_z;
            DD(i,:)=(((G_UC+G_R)*PPSI*SDX2).*((-G_R+B(i,:)-G_pi)*PPSI*SDX2));
        end
        
        B(:,end)=B(:,end)+A;
        B=B(2:end,:);
        B(:,end)=B(:,end)-pss*[1:400]';
        
        Ar=zeros(41,1); Br=zeros(41,NY);
        for i=1:40
            Ar(i+1) = -rbar + Ar(i) + (1/2)*(-G_R + Br(i,:))*PPSI*SDX2*SDX2'*PPSI'*(-G_R + Br(i,:))';
            Br(i+1,:) = -G_R*PHI + G_b + Br(i,:)*PHI ...
                + (G_UC+G_R)*PPSI*SDX2*SDX2'*PPSI'*(-G_R+Br(i,:))'*G_z;
        end
        
        Br(:,end)=Br(:,end)+Ar;
        Br=Br(2:end,:);
        
        aa=((RHO-ALPHA)/(1-RHO))*(G_UC+G_R)*PPSI*SDX2 - G_R*PPSI*SDX2;
        aa=(aa.^2)./sum(aa.^2);
%          disp((aa(2)+aa(9)));
        if (aa(2)+aa(9))<.9; eu=[0;0]; end
        
 else
     B=0;
     Br=0;
 end
 
 %% check for a reasonable SDF

 %
 
 
% disp(counter);
% Matrix that maps endogenous variables to observables 
% -------------------------------------------------------------------------
zmat=zeros(12,size(G1,2)); 
zmat(1,y)=1; zmat(1,y_1)=-1; zmat(1,a)=1; 
zmat(2,c)=1; zmat(2,c_1)=-1; zmat(2,a)=1;
zmat(3,x)=1; zmat(3,x_1)=-1; zmat(3,a)=1;
zmat(4,L)=1; 
zmat(5,w)=1; zmat(5,w_1)=-1; zmat(5,a)=1;
zmat(6,p)=1;
zmat(7,R)=1;

% constants in the observation equation 
% -------------------------------------------------------------------------
rss=1/LAMss-1;

% pbss2 = pbLss2;

C=zeros(11,1);
C(1)=gamma;          % D.gdp
C(2)=gamma;          % D.c
C(3)=gamma;          % D.i
C(4)=Lss;        % L
C(5)=gamma;          % D.w
C(6)=pss;            % inf
C(7)=pss+rss;        % interest rate

if ~isreal(PSI) | EX<1; eu=[0;0]; end

if sum(eu)==2; failmode=0; end;
beta=100/(Fbeta+100);

rbar=-log(beta) + RHO*gamma;


