%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Date: December 22, 2009
% This is the main code that controls the estimation of the baseline DSGE 
% model in "Investment Shocks and Business Cycles," by 
% Justiniano, Primiceri and Tambalotti
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% cd ~/fin_frictions/matlab/bonds
% addpath ..
clear all; rng shuffle; 
% addpath ~/fin_frictions/matlab/JPT_files

cd '~/FinFrictions/bonds'
addpath ..


warning('off','MATLAB:nearlySingularMatrix')
warning('off','MATLAB:illConditionedMatrix')

load data_bonds
T=length(y);

%% get initial points for the genetic algorithm
Large=33;

count=1;
tic
while count<3*Large+1
%     disp(count);
    xa=random_full14_bonds();
%     disp(logprior_full13(xa))
%     if logprior_full6(xa)<-5000; break; end;
    x0=bounds14_bondsINV(xa);
    j=logpost_bonds_14(x0,T,y); if j<1e19; disp(j); end
    if j<1e9
        InitPop(count,:)=x0;
        scores(count)=j;
        count=count+1;
    end
end
toc

IS=[scores' InitPop];
IS=sortrows(IS,1);
scores=IS(1:Large,1)';
InitPop=IS(1:Large,2:end);


POP=Large;

loss=@(x) logpost_bonds_14(x,T,y);
addpath ~idew/bond_pricing/JPT_alt/archive/ga_priv

%% initial hessian for the optimization; based on the standard deviations
%% in the prior
hess = ([.1,.1,.1,.1,8,.3,.03,.03,...
    .2*ones(1,8),1,2,1,1,1,1,.003,.08,.2,.2*ones(1,2),.1,5,.01,.2,.2,.2]);

%% genetic algorithm

myfun=@(opt,newx,newfval) rand(1)<exp((-newfval+opt.fval)/opt.temperature)

% PA = 1-exp((-newfval+opt.fval)/opt.temperature)
% dPA/dnew = 1/opt.temp

tic
[xh,fval,exitFlag,output,population,scores]=ga(loss,length(hess),[],[],[],[],[],[],[],...
    gaoptimset('display','iter','UseParallel','never','InitialPopulation',InitPop(1:POP,:),...
    'InitialScores',scores(1:POP),'PopulationSize',POP,...
    'generations',20,...
    'MutationFcn',{@mutationgaussianJPT, hess, 1}));
toc

xh=simulannealbnd(loss,xh,[],[],saoptimset('display','iter',...
                    'InitialTemperature',50,'maxiter',50,...
                    'AnnealingFcn',@annealingfast_pro,...
                    'PlotFcns',@saplotf))

xh=fminsearch(@(x) logpost_bonds_14(x,T,y),xh,optimset('display','iter','maxfunevals',50))


% high curvature=small stdev=large hessian=small inverse hessian, so the inverse hessian is proportional to the standard errors
hess = ([.1,.1,.1,.1,8,.3,.03,.03,...
    .2*ones(1,8),1,2,1,1,1,1,.003,.08,.2,.2*ones(1,2),.1,5,.01,.2,.2,.2]);

hess=.1*diag(hess/mean(hess));

%% save initial output
% fh=logpost_bonds_14(xh,T,y); xout=[fh xh]; save -ascii -append XOUT14_bonds.txt xout


%%%%%%%%%%%%%
%% switch to csminwel, fminsearch
tic
[fh,xh,gh,hess,itct,fcount,retcodeh] = csminwel('logpost_bonds_14',xh,hess,[],50e-4,1000,T,y);
toc

fh=logpost_bonds_14(xh,T,y); xout=[fh xh]; save -ascii -append XOUT14_bonds.txt xout

xh=simulannealbnd(loss,xh,[],[],saoptimset('display','iter',...
                    'InitialTemperature',5,'maxiter',200,...
                    'ReannealInterval',1000,...
                    'AnnealingFcn',@annealingfast_pro,...
                    'TemperatureFcn',@temperatureboltz))

                
tic
[fh,xh,gh,hess,itct,fcount,retcodeh] = csminwel('logpost_bonds_14',xh,hess,[],50e-4,1000,T,y);
toc


fh=logpost_bonds_14(xh,T,y); xout=[fh xh]; save -ascii -append XOUT14_bonds.txt xout

xh=fminsearch(@(x) logpost_bonds_14(x,T,y),xh,optimset('display','iter','maxfunevals',500))
xh=fminsearch(@(x) logpost_bonds_14(x,T,y),xh,optimset('display','iter','maxfunevals',200))

fh=logpost_bonds_14(xh,T,y); xout=[fh xh]; save -ascii -append XOUT14_bonds.txt xout

[fh,xh,gh,hess,itct,fcount,retcodeh] = csminwel('logpost_bonds_14',xh,hess,[],50e-4,200,T,y);

fh=logpost_bonds_14(xh,T,y); xout=[fh xh]; save -ascii -append XOUT14_bonds.txt xout


xh=fminsearch(@(x) logpost_bonds_14(x,T,y),xh,optimset('display','iter','maxfunevals',500))
xh=fminsearch(@(x) logpost_bonds_14(x,T,y),xh,optimset('display','iter','maxfunevals',500))

fh=logpost_bonds_14(xh,T,y); xout=[fh xh]; save -ascii -append XOUT14_bonds.txt xout

[fh,xh,gh,hess,itct,fcount,retcodeh] = csminwel('logpost_bonds_14',xh,hess,[],50e-4,1000,T,y);

fh=logpost_bonds_14(xh,T,y); xout=[fh xh]; save -ascii -append XOUT14_bonds.txt xout

[fh,xh,gh,hess,itct,fcount,retcodeh] = csminwel('logpost_bonds_14',xh,hess,[],50e-4,1000,T,y);

xh=fminsearch(@(x) logpost_bonds_14(x,T,y),xh,optimset('display','iter','maxfunevals',2000))
xh=fminsearch(@(x) logpost_bonds_14(x,T,y),xh,optimset('display','iter','maxfunevals',2000))

[fh,xh,gh,hess,itct,fcount,retcodeh] = csminwel('logpost_bonds_14',xh,hess,[],50e-4,1000,T,y);

fh=logpost_bonds_14(xh,T,y); xout=[fh xh]; save -ascii -append XOUT14_bonds.txt xout

%% odyssey code to run a big batch of jobs
% % % % % % % 
% cd ~/fin_frictions/matlab/bonds
% for x in {1..2000}
% do 
% bsub -q normal_serial -o /dev/null "matlab -nodisplay <optimize14_bonds.m >test.out" &
% done &

% "matlab -nodisplay <MCMC_fast.m >test_MCMC.out" &